# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_analyze_system.m
# input: AlphaL, AlphaU lower and upper arrival bound
#        Sys system description which is a structure with
#        the following fields which are distinguished according
#        to the filed Sys.type:
#        's' simple type with 
#            Sys.AlphaL, Sys.AlphaU, Sys.PhiL, Sys.PhiU
#            define the service
#        'c' sequential concatenation with
#            Sys.Subsys1, Sys.Subsys2
#            subsystems defining the concatenated services
#        'a' parallel and composition with
#            Sys.Subsys1, Sys.Subsys2
#            subsystems defining the parallel services
#        'o' parallel or composition with
#            Sys.p probability o choose Subsys 1
#            Sys.b maximal size of an arriving service call
#            Sys.Subsys1, Sys.Subsys2
#            subsystems defining the parallel services
#          
# output PhiL, PhiU lower and upper delay bounds
#        GammaL, GammU lower and upper output bounds
#
# description: The function computes results for a composed service
#              fed by some arrival process.
#
###########################################
function [PhiL, PhiU, GammaL, GammaU] = sla_analyze_system(AlphaL, AlphaU, Sys)

   # We start with the upper boundsdistinguish according to the type of system
   if (Sys.type == 's')
     [PhiL, PhiU, GammaL, GammaU] = sla_analyze_service(AlphaL, AlphaU, Sys.AlphaL, Sys.AlphaU, Sys.PhiL, Sys.PhiU) ;
   else
     if (Sys.type == 'c')
       [Phi1L, Phi1U, Gamma1L, Gamma1U] = sla_analyze_system(AlphaL, AlphaU, Sys.Subsys1) ;
       [PhiL, PhiU, GammaL, GammaU] = sla_analyze_system(Gamma1L, Gamma1U, Sys.Subsys2) ;
       PhiL = sla_add(PhiL, Phi1L) ;
       PhiU = sla_add(PhiU, Phi1U) ;
     else
       if (Sys.type == 'a')
       	 [Phi1L, Phi1U, Gamma1L, Gamma1U] = sla_analyze_system(AlphaL, AlphaU, Sys.Subsys1) ;
         [Phi2L, Phi2U, Gamma2L, Gamma2U] = sla_analyze_system(AlphaL, AlphaU, Sys.Subsys2) ;
	 Pd1 = sla_derivative(Phi1L) ;
	 Pd2 = sla_derivative(Phi2L) ;
	 Pd  = sla_max(Pd1, Pd2) ; 
	 PhiL = sla_integrate(Pd) ;
	 Pd1 = sla_derivative(Phi1U) ;
	 Pd2 = sla_derivative(Phi2U) ;
	 Pd  = sla_max(Pd1, Pd2) ; 
	 PhiU = sla_integrate(Pd) ;
	 GammaL = sla_min(Gamma1L, Gamma2L) ;
	 GammaU = sla_min(Gamma1U, Gamma2U) ;
       else
         # Sys.type == 'o' 
	 p1 = Sys.p ;
	 p2 = 1-p1 ;
	 bmax = Sys.b ;
	 Alpha1L = sla_add_constant(sla_mult_constant(AlphaL, p1), -p1*bmax, true)  ;
	 Alpha1U = sla_add_constant(sla_mult_constant(AlphaU, p1), p2*bmax, false)  ;
	 [Phi1L, Phi1U, Gamma1L, Gamma1U] = sla_analyze_system(Alpha1L, Alpha1U, Sys.Subsys1) ;
	 Alpha2L = sla_add_constant(sla_mult_constant(AlphaL, p2), -p2*bmax, true) ;
	 Alpha2U = sla_add_constant(sla_mult_constant(AlphaU, p2), p1*bmax, false) ;
	 [Phi2L, Phi2U, Gamma2L, Gamma2U] = sla_analyze_system(Alpha2L, Alpha2U, Sys.Subsys2) ;
	 Pd1 = sla_derivative(Phi1L) ;
	 Pd1 = sla_split_delay(Pd1, p1, p1*bmax) ;	 
	 Pd2 = sla_derivative(Phi2L) ;
	 Pd2 = sla_split_delay(Pd2, p2, p2*bmax) ;
	 Pd  = sla_min(Pd1, Pd2) ; 
	 PhiL = sla_integrate(Pd) ;
	 Pd1 = sla_derivative(Phi1U) ;
	 Pd1 = sla_split_delay(Pd1, p1, p1*bmax) ; 	 
	 Pd2 = sla_derivative(Phi2U) ;
	 Pd2 = sla_split_delay(Pd2, p2, p2*bmax) ; 
	 Pd  = sla_max(Pd1, Pd2) ; 
	 PhiU = sla_integrate(Pd) ;
	 # old version
	 # PhiL = sla_max(Phi1L, Phi2L) ;
	 # PhiU = sla_max(Phi1U, Phi2U) ;
	 GammaL = sla_min(sla_add(Gamma1L, Gamma2L), AlphaL) ;
	 Gamma1U
	 Gamma2U
	 sla_add(Gamma1U, Gamma2U)
	 GammaU = sla_min(sla_add(Gamma1U, Gamma2U), AlphaU) ;
       endif
     endif
   endif
   # reduce the resulting curves when necessary
   PhiL = sla_reduce(PhiL) ;
   PhiU = sla_reduce(PhiU) ;
   GammaL = sla_reduce(GammaL) ;
   GammaU = sla_reduce(GammaU) ;

endfunction
