# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_max_segment.m
# input: x1, y1, x2, y2 x and y indices of the two curves
#        fin1 , fin2 if true the curves are finite
#       
# output [xx, yy] segments of the maximum of the curve
#
###########################################
function [xx,yy] = sla_max_segment(x1, y1, fin1, x2, y2, fin2)


   if (sla_check_xy(x1, y1) && sla_check_xy(x2, y2))
      # check for the smallest value in the vectors
      # if it is positive choose zero as a lower bound else multiply it with 2
      small_value = min(min(min(min(x1), min(y1)),min(x2)), min(y2)) ;
      if (small_value >= 0)
         small_value = 0 ;
      else
         small_value = 10*small_value ;
      endif
      # a large value is required for the x-axis
      large_x_value = 10*max(max(x1), max(x2)) + 1 ;
      # if a segment does not start at 0 we add a part with small_values, 
      # if both are zero we add first a zero part
      if (x1(1) > 0 && x2(1) > 0)
         if (x1(1) < x2(1))
	    x0(1) = 0 ;
	    y0(1) = 0 ;
	    x0(2) = x1(1) ;
	    y0(2) = 0 ;
	    for (i=1:length(x1))
	       x0(i+2) = x1(i) ;
	       y0(i+2) = y1(i) ;
	    endfor
	    x1 = x0 ;
	    y1 = y0 ;
	    x0(1) = 0 ;
	    y0(1) = small_value ;
	    x0(2) = x2(1) ;
	    y0(2) = small_value ;
	    for (i=1:length(x2))
	       x0(i+2) = x2(i) ;
	       y0(i+2) = y2(i) ;
	    endfor
	    x2 = x0 ;
	    y2 = y0 ;
	 else
	    x0(1) = 0 ;
	    y0(1) = small_value ;
	    x0(2) = x1(1) ;
	    y0(2) = small_value ;
	    for (i=1:length(x1))
	       x0(i+2) = x1(i) ;
	       y0(i+2) = y1(i) ;
	    endfor
	    x1 = x0 ;
	    y1 = y0 ;
	    x0(1) = 0 ;
	    y0(1) = 0 ;
	    x0(2) = x2(1) ;
	    y0(2) = 0 ;
	    for (i=1:length(x2))
	       x0(i+2) = x2(i) ;
	       y0(i+2) = y2(i) ;
	    endfor
	    x2 = x0 ;
	    y2 = y0 ;
	 endif
      else
         if (x1(1) > 0)
	    x0(1) = 0 ;
	    y0(1) = small_value ;
	    x0(2) = x1(1) ;
	    y0(2) = small_value ;
	    for (i=1:length(x1))
	       x0(i+2) = x1(i) ;
	       y0(i+2) = y1(i) ;
	    endfor
	    x1 = x0 ;
	    y1 = y0 ;
	 endif
	 if (x2(1) > 0)
	    x0(1) = 0 ;
	    y0(1) = small_value ;
	    x0(2) = x2(1) ;
	    y0(2) = small_value ;
	    for (i=1:length(x2))
	       x0(i+2) = x2(i) ;
	       y0(i+2) = y2(i) ;
	    endfor
	    x2 = x0 ;
	    y2 = y0 ;
	 endif
      endif
      # if both are finite, we have to set the final segment with the larger 
      # x-value to 0 instead
      if (fin1 && fin2)
         if (x1(length(x1)) < x2(length(x2)))
            x1(length(x1)+1) = x1(length(x1)) ;
            y1(length(y1)+1) = small_value ;
            x1(length(x1)+1) = x1(length(x1)) + 1 ;
            y1(length(y1)+1) = small_value ;
            x2(length(x2)+1) = x2(length(x2)) ;
            y2(length(y2)+1) = 0.0 ;
            x2(length(x2)+1) = x2(length(x2)) + 1 ;
            y2(length(y2)+1) = 0.0 ;
         else
            x1(length(x1)+1) = x1(length(x1)) ;
            y1(length(y1)+1) = 0.0 ;
            x1(length(x1)+1) = x1(length(x1)) + 1 ;
            y1(length(y1)+1) = 0.0 ;
            x2(length(x2)+1) = x2(length(x2)) ;
            y2(length(y2)+1) = small_value ;
            x2(length(x2)+1) = x2(length(x2)) + 1 ;
            y2(length(y2)+1) = small_value ;
         endif
      else
         # if we have only one finite segemnt we add a small part to end 
         # which disappears in the myximum 
         if (fin1)
            x1(length(x1)+1) = x1(length(x1)) ;
            y1(length(y1)+1) = small_value ;
            x1(length(x1)+1) = x1(length(x1)) + 1 ;
            y1(length(y1)+1) = small_value ;
         endif
         if (fin2)
            x2(length(x2)+1) = x2(length(x2)) ;
            y2(length(y2)+1) = small_value ;
            x2(length(x2)+1) = x2(length(x2)) + 1 ;
            y2(length(y2)+1) = small_value ;
         endif
      endif
      # add a dummy segemnt with a large x value ot both curves
      x1(length(x1)+1) = large_x_value ;
      x2(length(x2)+1) = large_x_value ;
      i1 = 1 ;
      i2 = 1 ;
      s1 = 0 ;
      s2 = 0 ;
      j  = 1 ;
      while (i1 < length(x1) || i2 < length(x2))
         # determine the different cases where for the x-values
	 if (x1(i1) < x2(i2))
	    xnext = x1(i1) ;
	    z1 = y1(i1) ;
	    if (i2 < 2)
	       z2 = small_value ;
	    else
               z2 = (xnext - x2(i2-1)) * s2 + y2(i2-1) ;
	    endif
	    i1++ ;
	 else
            if (x1(i1) > x2(i2))
	       xnext = x2(i2) ;
	       z2 = y2(i2) ;
	       if (i1 < 2)
	          z1 = small_value ;
	       else
                  z1 = (xnext - x1(i1-1)) * s1 + y1(i1-1) ;
	        endif
               i2++ ;
            else
 	       xnext = x1(i1) ;
	       z1 = y1(i1) ;
	       z2 = y2(i2) ;
	       i1++ ;
               i2++ ;
	    endif
	 endif
         xx(j) = xnext ;
         yy(j) = max(z1, z2) ;
	 if (j == 1 || xx(j) != xx(j-1) || yy(j) != yy(j-1))
	   j++ ;
	 endif
         # determine the slopes which remains if the final x-segemnt has been reached
         if (i1 > 1 && i1 < length(x1))
	    # only if the next x-value is different, which means no jump,
            # the slope is computed otherwise it remains the same
            if (x1(i1) > x1(i1-1))
	       s1 = (y1(i1)-y1(i1-1))/(x1(i1)-x1(i1-1)) ;
           else
               s1 = 0 ;
	   endif
         endif
         if (i2 > 1 && i2 < length(x2))
	    # only if the next x-value is different, which means no jump,
            # the slope is computed otherwise it remains the same
            if (x2(i2) > x2(i2-1))
	       s2 = (y2(i2)-y2(i2-1))/(x2(i2)-x2(i2-1)) ;
            else
               s2 = 0 ;
	    endif
         endif
	 # Only if we have not reached teh final segement of a finite curve we
         # have to look for intersections
         if ((!(fin1) || i1 < length(x1) - 1) && (!(fin2) || i2 < length(x2) - 1))
	 # determine the following x-value and check whether the line segments intersect before
	    xfuture = min(x1(i1), x2(i2)) ;
	    if ((z1 < z2 && s1 > s2) || (z1 > z2 && s1 < s2))
	       xintersect = (z1-z2)/(s2-s1) + xnext ;
 	       if (xintersect < xfuture)
	       	  xx(j) = xintersect ;
	       	  yy(j) = (xintersect - xnext)*s1 + z1 ;
	       	  if (j == 1 || xx(j) != xx(j-1) || yy(j) != yy(j-1))
	             j++ ;
	       	  endif
	       endif 
	    endif
	 endif
      endwhile
   else
     printf("Sorry x and y coordinates are not consistent\n") ;
   endif
endfunction