# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_min
#
# input: A, B matrices of the type
#        A {a1 a2 an}
#        where ai = [xi, yi, si]
#
# output C = min(A, B)
#
###########################################
function C = sla_min(A, B)

   A = sla_add_zero(A) ;
   B = sla_add_zero(B) ;
   # We can be sure that both curves start at 0
   if (A{1}(2) < B{1}(2))
      s = A{1}(3) ;
   else
      if (A{1}(2) > B{1}(2))
         s = B{1}(3) ;
      else
         # both start with the same  y value
         s = min(A{1}(3), B{1}(3)) ;
      endif
   endif
   x = xnext = 0 ;
   y = min(A{1}(2),B{1}(2)) ;
   # the first segment
   C = {[xnext, y, s]} ;
   # i1 index of A i2 index of B j index of C
   i1 = 1 ;
   i2 = 1 ;
   j  = 2 ;
   while (true)
      x1 = A{i1}(1) ;
      x2 = B{i2}(1) ;
      s1 = A{i1}(3) ;
      s2 = B{i2}(3) ;
      y1 = (xnext - x1) * s1 + A{i1}(2) ;
      y2 = (xnext - x2) * s2 + B{i2}(2) ;
      if (i1 < columns(A))
         if (i2 < columns(B))
	    if (A{i1+1}(1) < B{i2+1}(1)) 
	       xnext = A{i1+1}(1) ;
	       ind = 1 ;
	    else
	       xnext = B{i2+1}(1) ;
	       ind = 2 ;
	    endif
	    y1next = (xnext - x1) * s1 + A{i1}(2) ;
	    y2next = (xnext - x2) * s2 + B{i2}(2) ;
	 else
	    xnext = A{i1+1}(1) ;
	    ind = 1 ;
	    y1next = (xnext - x1) * s1 + A{i1}(2) ;
	    y2next = (xnext - x2) * s2 + B{i2}(2) ;
	 endif
      else
         if (i2 < columns(B))
	    xnext = B{i2+1}(1) ;
	    ind = 2 ;
	    y1next = (xnext - x1) * s1 + A{i1}(2) ;
	    y2next = (xnext - x2) * s2 + B{i2}(2) ;
	 else
	    ind = 0 ;
	    xnext = 1.0e+12 ;
	 endif
      endif
      if ((y1 <= y2 && s1 > s2) || (y1 >= y2 && s1 < s2))
         xnew = (y1 - y2) /(s2 - s1) + x ;
	 ynew = y1 + min(s1,s2)*(xnew - x) ;
	 s = min(s1,s2) ;   
	 if (xnew < xnext)
	    C{j} = [xnew, ynew, s] ;
	    x = xnew ;
	    j++ ;
	 endif
      endif
      if (ind == 0)
         # Check whether there is a crossing point in the last segements
	 xnext = max(A{i1}(1), B{i2}(1)) ;
	 y1 = (xnext - A{i1}(1)) * A{i1}(3) + A{i1}(2) ;
	 y2 = (xnext - B{i2}(1)) * B{i2}(3) + B{i2}(2) ;
	 s1 = A{i1}(3) ;
	 s2 = B{i2}(3) ; 
         if ((y1 < y2 && s1 > s2) ||
	     (y1 > y2 && s1 < s2)   )
	    # compute the crossing point of the two curves
	    xnew = (y1 - y2) / (s2 - s1) + xnext ;
	    ynew = y1 + max(s1,s2)*(xnew - xnext) ;
	    s = min(s1,s2) ;  
	    if (xnew > xnext) 
	       C{j} = [xnew, ynew, s] ;
	       x = xnew ;
	       j++ ;
	    endif
	 endif
      	 C = sla_reduce(C) ;
         return ;
      endif
      if (ind == 1)
         i1++ ;
         y1next = A{i1}(2) ;	
	 s1 = A{i1}(3) ;
      else
         i2++ ;
	 y2next = B{i2}(2) ;
	 s2 = B{i2}(3) ;
      endif
      if (y1next < y2next)
      	 s = s1 ;
      else
         if (y1next > y2next)
            s = s2 ;
	 else
            s = min(s1, s2) ;
	 endif
      endif
      C{j} = [xnext, min(y1next, y2next), s] ;
      x = xnext ;
      j++ ;
   endwhile
endfunction
